Index: chan_sip.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channels/chan_sip.c,v
retrieving revision 1.705
diff -u -r1.705 chan_sip.c
--- chan_sip.c  21 Apr 2005 06:02:43 -0000      1.705
+++ chan_sip.c  26 Apr 2005 19:26:52 -0000
@@ -2099,6 +2099,8 @@
        return res;
 }

+#define DEFAULT_MAX_FORWARDS   70
+

 /*--- sip_transfer: Transfer SIP call */
 static int sip_transfer(struct ast_channel *ast, const char *dest)
@@ -4621,6 +4623,7 @@
        char from[256];
        char *of, *c;
        char referto[256];
+       char tmp[80];

        if (ast_test_flag(p, SIP_OUTGOING))
                of = get_header(&p->initreq, "To");
@@ -4634,7 +4637,9 @@
        } else
                of += 4;
        /* Get just the username part */
-       if ((c = strchr(of, '@'))) {
+       if ((c = strchr(dest, '@'))) {
+               c = NULL;
+       } else if ((c = strchr(of, '@'))) {
                *c = '\0';
                c++;
        }
@@ -4644,16 +4649,26 @@
                snprintf(referto, sizeof(referto), "<sip:%s>", dest);
        }

-       /* save in case we get 407 challenge */
-       strncpy(p->refer_to, referto, sizeof(p->refer_to) - 1);
-       strncpy(p->referred_by, p->our_contact, sizeof(p->referred_by) - 1);
-
-       reqprep(&req, p, SIP_REFER, 0, 1);
-       add_header(&req, "Refer-To", referto);
-       if (!ast_strlen_zero(p->our_contact))
-               add_header(&req, "Referred-By", p->our_contact);
-       add_blank_header(&req);
-       return send_request(p, &req, 1, p->ocseq);
+       strncpy(tmp, get_header(&p->initreq, "Max-Forwards"), sizeof(tmp) - 1);
+       if (strlen(tmp) && atoi(tmp)) {
+               p->maxforwards = atoi(tmp) - 1;
+       } else {
+               p->maxforwards = DEFAULT_MAX_FORWARDS - 1;
+       }
+       if (p->maxforwards > -1) {
+               /* save in case we get 407 challenge */
+               strncpy(p->refer_to, referto, sizeof(p->refer_to) - 1);
+               strncpy(p->referred_by, p->our_contact, sizeof(p->referred_by) - 1);
+
+               reqprep(&req, p, SIP_REFER, 0, 1);
+               add_header(&req, "Refer-To", referto);
+               if (!ast_strlen_zero(p->our_contact))
+                       add_header(&req, "Referred-By", p->our_contact);
+               add_blank_header(&req);
+               return send_request(p, &req, 1, p->ocseq);
+       } else {
+               return -1;
+       }
 }

 /*--- transmit_info_with_digit: Send SIP INFO dtmf message, see Cisco documentation on cisco.co
@@ -10629,8 +10644,6 @@
        ast_mutex_unlock(&chan->lock);
        return 0;
 }
-
-#define DEFAULT_MAX_FORWARDS   70

 /*--- sip_sipredirect: Transfer call before connect with a 302 redirect ---*/
 /* Called by the transfer() dialplan application through the sip_transfer() */
